function [C, in]=hcdft(B,tflag,in)
%
% Computes hyperbolic cross dft and its adjoint
%
% [C,in]=hcdft(B,tflag,in)
%
% B     = [if tflag='notransp']
%         fourier coefficients over hyperbolic cross (vector or tensor)
%         [if tflag='transp']
%         function values over sparse grid (vector or tensor)
% tflag = 'notransp' evaluate trigonometric polynomial
%         'transp' adjoint transform
% in    = with hcfft_initialize(d,n) produced object or
%         easily vector [d,n] -> hcdft calls hcfft_initialize(d,n)
%         optional, if B is a tensor
%
% Example:
%   
%   d=4; n=3;                             % dimension 4, level 3
%
%   in =hcfft_initialize(d,n);            % in can also be in form in=[d,n];
%
%   B=rand(in.s(d,n+1),1);                % 5 fourier coefficients sequences
%
%   C=hcdft(B, 'notransp', in);           % computes function values over
%                                         % the sparse grid
%
% See also: hcfft, nhcdft, nhcfft
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: hcdft.m 536 2009-05-06 13:20:30Z kaemmerer $


    % maybe B is a vector or a tensor
    try
        % B.size doesn't exist for simple vector but for tensor
        B.size;
        tensor=1;
    catch
        tensor=0;
    end

    
    % hcfft_initialize in, if it's not there
    if tensor==0
        % maybe in is [d,n]
        switch length(in)
            case 1
                % ok, in is ready
                [d,n]=size(in.s);
                n=n-1;
            case 2
                d=in(1);
                n=in(2);
                in=hcfft_initialize(d,n);
            otherwise
                disp('problem with argument "in"');
                return;
        end
    else
        
        Nv=size(B);
        d=length(Nv);
        n=round(log2(Nv(1)));
        if(any(Nv~=2^n))
            disp('Size of input matrix (tensor) must be the same power of 2 in each dimension');
            return;
        end

        switch nargin
            case 3
                % in must be computed from B
                in=hcfft_initialize(d,n);
            case 4
                switch length(in)
                    case 1
                        % look, if B has the same size like in
                        [d_in,n_in]=size(in.s);
                        n_in=n_in-1;
                        if(n_in~=n || d_in~=d)
                            disp('in and B are not compatible');
                            return;
                        end
                    case 2
                        if(d~=in(1) || n~=in(2))
                            disp('in and B are not compatible');
                            return;
                        else
                            in=hcfft_initialize(d,n);
                        end
                    otherwise
                        disp('problem with argument "in"');
                        return;
                end
            otherwise
                disp('problem with number of arguments');
                return;
        end
        
    end
    
    if tensor==1 && strcmp(tflag, 'notransp')
        B=hcfft_hc2vec(B,in);
    elseif tensor==1 && strcmp(tflag, 'transp')
        B=hcfft_sg2vec(B,in);
    end
    
  
    switch tflag
        case 'notransp'
            C=hcdft_notransp(B,d,n,in);
        case 'transp'
            C=ahcdft(B,d,n,in);
        otherwise
            disp('Problem with tflag');
            return;
    end
    
    
    if tensor==1 && strcmp(tflag, 'notransp')
        C=hcfft_vec2sg(C,d,n,in);
    elseif tensor==1 && strcmp(tflag, 'transp')
        C=hcfft_vec2hc(C,d,n,in);
    end
    
end


function [C, in] = hcdft_notransp(B, d, n, in)
%Computes function values on the sparse grid out of the Fourier coefficents
%on the hyperbolic cross by computing the naive sum. Input vector B
%contains the Fourier coefficients, output vector C the sparse grid. 

%in (struct with some initial matrices) is an optional argument

    if(nargin==3)
        in=hcfft_initialize(d,n);
        disp('initialization in hcdft');
    end
    
    hc=hcfft_coordhc(d, n, in);
    sg=hcfft_coordsg(d, n, in);

    N=in.s(d,n+1);
    if(N~=length(B))
        disp('length of input vector does not fit to dimension d and level n');
        return;
    end
    b=size(B);
    C=zeros(1,N);
    for j=1:N
        C = C + B(j)*exp(2*pi*1i*(hc(j,:)*sg'));
    end
    if(b(1)>1)
        C=C.';
    end
end


function [C, in] = ahcdft(B, d, n, in)
%Computes adjoint HCDFT (slow!). Input vector B contains the Fourier
%coefficients, output vector C the sparse grid. 

%in (struct with some initial matrices) is an optional argument

    if(nargin==3)
        in=hcfft_initialize(d,n);
        disp('initialization in hcdft');
    end
    
    hc=hcfft_coordhc(d, n, in);
    sg=hcfft_coordsg(d, n, in);

    N=in.s(d,n+1);
    if(N~=length(B))
        disp('length of input vector does not fit to dimension d and level n');
        return;
    end
    b=size(B);
    C=zeros(1,N);
    for j=1:N
        C = C + B(j)*exp(-2*pi*1i*(sg(j,:)*hc'));
    end
    if(b(1)>1)
        C=C.';
    end
end
