function [B in]=hcfft_sg2vec(C, in)
%
% Converts sg tensor to sg vector in order of the indexing function
%
% Reads sparse grid entries from tensor or sparse tensor C in any 
% dimensions and writes them into array B
%
% [B in] = hcfft_sg2vec(C,in)
%
% C     = sparse grid as (sparse) tensor
% in    = (struct with some initial matrices) is an optional argument
%
% Example:
%
%   in=hcfft_initialize(2,3);
%   s_vec=rand(in.s(2,4),1);        % function values over sg as vector
%   s_ten=hcfft_vec2sg(s_vec,in);   % sg as sparse tensor (possibly easier handling)
%   s_ten(5,5)=0;                   % manipulate the function values
%                                   % at (0.5,0.5)~(1+2^(n-1),1+2^(n-1))
%   s_vec1=hcfft_sg2vec(s_ten,in);  % sg as vector for computation
%
% See also: hcfft_vec2sg, hcfft_coordsg
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: hcfft_sg2vec.m 544 2009-05-13 09:17:01Z kaemmerer $

    x=size(C);
    d=length(x);
    n=round(log2(x(1)));
    if(any(x~=2^n))
        disp('length of any dimension must be equal and a power of 2');
        return;
    end
    if(nargin==1)
        in=hcfft_initialize(d,n);
        %disp('initialization in hcfft_sg2vec');
    end
    B=zeros(in.s(d, n+1),1);
    l=zeros(1,d);
    global HCFFT_C HCFFT_B;
    HCFFT_C=C;
    HCFFT_B=B;
    for k=0:n
        numl(l, 1, d, k, n, in);
    end
    B=HCFFT_B;
    clear HCFFT_C HCFFT_B;
end

function v = pos(l,q,n,d)
%returns the position of the point (l,q) in the sparse grid matrix
    for i=1:d
        v(i)=2^(n-l(i))*q(i)+1;
    end
end

function numl(l, d1, d, k, n, in)
    m=0;
    if(d1>1)
       m=sum(l(1:d1-1));
    end
    for i=0:k-m
        l(d1)=i;
        if(d1<d-1)
            numl(l, d1+1, d, k, n, in);
        else
            l(d)=k-m-i;
            q=zeros(1,d);
            numq(q, 1, d, l, n, k, in);
        end 
    end
end

function numq(q, d1, d, l, n, k, in)
    for i=1:ceil(2^(l(d1)-1))
        if(l(d1)==0)
            q(d1)=0;
        else
            q(d1)=2*i-1;
        end
        if(d1<d)
            numq(q, d1+1, d, l, n, k, in);
        else
            global HCFFT_C HCFFT_B;
            ind=hcfft_index(l, q, d, k, in);
            v = pos(l, q, n, d);
            HCFFT_B(ind)=HCFFT_C(v);
        end
    end            
end
