function [C,in]=nhcdft(B,x,tflag,in)
%
% Computes nonequispaced hyperbolic cross dft and its adjoint
%
% [C,in] = nhcdft(B,x,tflag,in)
%
% B     = [if tflag='notransp']
%         fourier coefficients over hyperbolic cross H_d^n (vector or tensor)
%         [if tflag='transp']
%         function values over points x (vector)
% x     = points, where function-values should be computed or are given
% tflag = 'notransp' => nhcdft
%         'transp' => adjoint algorithm
% in    = object from hcfft_initialize(d,n) or
%         simple vector [d,n] -> hcfft_initialize will be called from
%         nhcdft
%         optional, if B is a tensor
%
% Example:
%
%   d=3; n=4; in=hcfft_initialize(d,n);
%
%   B=rand(in.s(d,n+1),1);
%   B=hcfft_vec2hc(B,d,n,in);   % sequence of fourier coefficients
%
%   x=rand(100,d);              % sequence of arbitrary points of T^d
%
%   C=nhcdft(B,x,'notransp',in);    % function values C(j)=f(x(j))
%
% See also: nhcfft, hcdft, hcfft
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: nhcdft.m 544 2009-05-13 09:17:01Z kaemmerer $

    % maybe B is a vector or a tensor
    try
        % B.size exists only for tensors
        Nv=B.size;
        d=length(Nv);
        n=round(log2(Nv(1)));
        if(any(Nv~=2^n))
            disp('Size of input matrix must be the same power of 2 in each dimension');
            return;
        end
        tensor=1;
    catch
        tensor=0;
    end

    % test and initialize in
    if (tensor==1 && nargin==3) || (tensor==1 && length(in)==2 && isequal(in,[d,n]))
        in=hcfft_initialize(d,n);
        B=hcfft_hc2vec(B,in);
    elseif tensor==1 && length(in)==1 && isequal(size(in.s),[d,n+1])
        B=hcfft_hc2vec(B,in);
    elseif tensor==0 && length(in)==2
        d=in(1); n=in(2);
        in=hcfft_initialize(d,n);
    elseif tensor==0 && length(in)==1
        [d,n]=size(in.s);
        n=n-1;
    else
        disp('problem with in');
        return;
    end
       
    switch tflag
        case 'transp'
            [C,in]=anhcdft(B,d,n,x,in);
        case 'notransp'
            [C,in]=nhcdft_loop(B,d,n,x,in);
        otherwise
            disp('problem with argument tflag');
            return;
    end

end


function [C in] = nhcdft_loop(B, d, n, x, in)
%Computes function values of trigonmetric polynomial given by hyperbolic
%cross in vector B of dimension d and level n at the points of vector x by
%computing the naive sum. size(x)=[M d] i.e. each row of x contains a point
%at which the polynomial will be evaluated. 

%in (struct with some initial matrices) is an optional argument

    s=size(x);
    if(s(2)~=d)
        disp('The M points at which the trigonometric polynomial will be evaluated must be in a matrix of the size M x d.')
        return;
    end
    M=s(1);
    if(nargin==4)
        in=hcfft_initialize(d,n);
        disp('initialization in nhcdft');
    end
    
    hc=hcfft_coordhc(d, n, in);

    N=in.s(d,n+1);
    if(N~=length(B))
        disp('length of input vector does not fit to dimension d and level n');
        return;
    end
    b=size(B);
    C=zeros(1,M);
    for j=1:N
        C = C + B(j)*exp(2*pi*1i*(hc(j,:)*x'));
    end
    if(b(1)>1)
        C=C.';
    end
end


function [C in] = anhcdft(B, d, n, x, in)
%Computes fourier coefficients of trigonmetric polynomial given by function
%values in vector B of dimension d and level n at the points of vector x by
%computing the naive sum. size(x)=[M d] i.e. each row of x contains a point
%at which the polynomial will be evaluated. 

%in (struct with some initial matrices) is an optional argument

    s=size(x);
    if(s(2)~=d)
        disp('The M points at which the trigonometric polynomial will be evaluated must be in a matrix of the size M x d.')
        return;
    end
    M=s(1);
    if(nargin==4)
        in=hcfft_initialize(d,n);
        disp('initialization in nhcdft');
    end
    
    hc=hcfft_coordhc(d, n, in);

    N=in.s(d,n+1);
    if(s(1)~=length(B))
        disp('length of input vector B does not fit to length of x');
        return;
    end
    b=size(B);
    C=zeros(1,M);
    for j=1:N
        C = C + B(j)*exp(-2*pi*1i*(x(j,:)*hc'));
    end
    if(b(1)>1)
        C=C.';
    end
end
