function [ a ]=nhcfft_bool_int( T, m, tflag, in)
% Interpolation of values on vector T containing a sg with periodic splines of order m
%
% [a]=nhcfft_bool_int(T,m,tflag,in)
%
% T     = vector containing a sparse grid (1d, 2d or 3d)
% m     = order of periodic spline (2,4,6,8,10,12,14 or 16)
% tflag = 'notransp' computes spline coefficients
%         'transp' adjoint computation
% in    = result from hcfft_initialize function or
%         simple vector [d,n], d = dimension, n=level of hc
%
% function is only for d=1, d=2 and d=3
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: nhcfft_bool_int.m 536 2009-05-06 13:20:30Z kaemmerer $

    if length(in)==1
        [d,n]=size(in.s);
        n=n-1;
    elseif length(in)==2
        d=in(1); n=in(2);
        in=hcfft_initialize(d,n);
    else
        disp('problem with argument in');
        return;
    end


    p=zeros(1,m+1);
    for k=1:m+1
        p(k)=2^(1-m)*nchoosek(m,k-1);
    end

    switch tflag
        case 'notransp'
            
            switch d
                case 1
                    a=spline_int(T,m);
                case 2
                    a=bool_int_2d(T,m,n,p,in);
                case 3
                    a=bool_int_3d(T,m,n,p,in);
                otherwise
                    disp('dimension d has to be two or three');
                    return;
            end
            
        case 'transp'
            switch d
                case 1
                    a=aspline_int(T,m);
                case 2
                    a=abool_int_2d(T,m,n,p,in);
                case 3
                    a=abool_int_3d(T,m,n,p,in);
                otherwise
                    disp('For adjoint computation dimension d has to be two or three.');
            end
        otherwise
            disp('Only "transp" or "notransp" is a valid use for input argument tflag.');
    end
end


function [a]=bool_int_2d(T,m,n,p,in)

    % tests length only if function is not called from the 3d-case
    if(nargin==5 && length(T)~=(n+2)*2^(n-1))
        disp('length of input vector must be like (n+2)*2^(n-1)');
        return;
    end
    
    m2=m/2;
    
    if(sum(ismember([1,2,3,4,5,6,7,8],m2))==0)
        disp('Order of spline has to be 2, 4, 6, 8, 10, 12, 14 or 16');
        return;
    end
    
    a=zeros(1,(n+1)*2^n);
    w=a;
    u=a;
    v=a;
    
    % change format only if in is an argument, otherwise T comes from the
    % 3d-case and is in format we need
    if (nargin==5)
    % step 0: change format of sparse grid vector to our needs
    
        for k=0:n   
            for s=k:-1:0
                % all level indices l
                l(1)=s;
                l(2)=k-s;  
                for ii=1:ceil(2^(l(1)-1))
                    for jj=1:ceil(2^(l(2)-1))
                        % all space indices for every level
                        if(l(1)==0)
                            q(1)=0;
                        else
                            q(1)=2*ii-1;
                        end
                        if(l(2)==0)
                            q(2)=0;
                        else
                            q(2)=2*jj-1;
                        end
                        ind=hcfft_index(l, q, 2, k, in);
                        for kk=l(1):n-l(2)
                            w(isp(n, kk, 2^(kk-l(1))*q(1),2^(n-kk-l(2))*q(2)))=T(ind);
                        end
                    end
                end
            end
        end
    else
        w=T;
    end
    
    
    %step 1
    for r=0:n
        for s=0:r
            for t=0:(ceil(2^(s-1))-1)
                if(s==0)
                    q=0;
                else
                    q=2*t+1;
                end
                mu=q*2^(r-s);
                i1=isp(n,r,mu,0);
                u(i1 : i1-1+2^(n-r))=spline_int(w(i1 : i1-1+2^(n-r)),m);
            end
        end
    end
    
   
    %step 2
    for r=0:n-1
        for mu=0:2^r-1
            for l=0:2^(n-r-1)-1
                i1=isp(n,r,mu,2*l);
                v(i1) = u(i1);
                v(i1+1) = u(i1+1);
                for k=0:m2-1
                    i2=isp(n,r+1,2*mu,mod(l-k,2^(n-r-1)));
                    v(i1) = v(i1) - p(2*k+1)*u(i2);
                    v(i1+1) = v(i1+1) - p(2*k+2)*u(i2);
                end
                v(i1) = v(i1) - p(m+1)*u(isp(n,r+1,2*mu,mod(l-m2,2^(n-r-1))));
            end
        end
    end
    v(n*2^n+1:end)=u(n*2^n+1:end);
    
    %step 3
    for r=0:n
        for l=0:2^(n-r)-1
            i1=isp(n,r,0,l);
            i2=isp(n,r,2^r-1,l);
            IND=i1:2^(n-r):i2;
            a(IND)=spline_int(v(IND),m);
        end
    end

    [s1 s2]=size(T);
    if(s2==1)
        a=a.';
    end
end


function [ a ] = bool_int_3d( T, m, n, p, in)
%Interpolation of values on vector T containing a 3d sparse grid with
%periodic splines of degree m 

    if(length(T)~=2^n*(n+1)+2^(n-3)*n*(n-1))
        disp('length of input vector must be like 2^k*(k+1)+2^(k-3)*k*(k-1)');
        return;
    end

    m2 = m/2;
    if(sum(ismember([1,2,3,4,5,6,7,8],m2))==0)
        disp('Order of spline must be 2, 4, 6, 8, 10, 12, 14 or 16');
        return;
    end
    a=zeros(1,(n+1)*(n+2)*2^(n-1));
    w=a;
    u=a;
%    v=a;
%    u1=zeros(1,m2+1);

    %step 0: change format of sparse grid vector to our needs
    for k=0:n  
        for a=0:k
            for b=0:(k-a)
                l(1)=a;
                l(2)=b;
                l(3)=k-a-b;
                for i1=1:ceil(2^(l(1)-1))     %Durchlaufen aller Punkte zum Level l
                    for i2=1:ceil(2^(l(2)-1))
                        for i3=1:ceil(2^(l(3)-1))
                            if(l(1)==0)
                                q(1)=0;
                            else
                                q(1)=2*i1-1;
                            end
                            if(l(2)==0)
                                q(2)=0;
                            else
                                q(2)=2*i2-1;
                            end
                            if(l(3)==0)
                                q(3)=0;
                            else
                                q(3)=2*i3-1;
                            end
            
                            ind=hcfft_index(l, q, 3, k, in);
                            for r3=l(3):n-l(1)-l(2)
                                for r2=l(2):n-l(1)-r3
                                    r1=n-r2-r3;
                                    w(isp3(n, r1+r2, r1, 2^(r1-l(1))*q(1), 2^(r2-l(2))*q(2), 2^(r3-l(3))*q(3)))=T(ind);
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
    %step 1
    u(1:2^n)=spline_int(w(1:2^n), m);
    for j=1:n
        i1=j*(j+1)*2^(n-1);
        for jj=0:(j+1)*2^j-1
            u(i1+jj*2^(n-j)+1:i1+(jj+1)*2^(n-j)) = spline_int(w(i1+jj*2^(n-j)+1:i1+(jj+1)*2^(n-j)), m);
        end
    end
    
    %step 2
    v=u;
    for j=0:n-1
        for j1=0:j
            for k=0:2^j1-1
                for l=0:2^(j-j1)-1
                    for mm=0:2^(n-j-1)-1
                        i1=isp3(n, j, j1, k, l, 2*mm);
                        for s=0:m2-1
                            i2=isp3(n, j+1, j1, k, 2*l, mod(mm-s, 2^(n-j-1)));
                            v(i1) = v(i1) - p(2*s+1)*u(i2);
                            v(i1+1) = v(i1+1) - p(2*s+2)*u(i2);
                        end
                        v(i1) = v(i1) - p(m+1)*u(isp3(n, j+1, j1, k, 2*l, mod(mm-m2, 2^(n-j-1))));
                    end
                end
            end
        end
    end
    
    %step 3
    for j=0:n
        for mm=0:2^(n-j)-1
            i1=isp3(n, j, 0, 0, 0, mm);
            i2=isp3(n, j, j, 2^j-1, 0, mm);
            IND=i1:2^(n-j):i2;
            a(IND)=bool_int_2d(v(IND), m, j, p);
        end
    end

    [s1 s2]=size(T);
    if(s2==1)
        a=a.';
    end
end

function [ S ] = abool_int_2d( a, m, n, p, in )
%Interpolation of values on sparse grid S with periodic splines of degree m
%adjoint algorithm in 2d

   
    if(length(a)~=(n+1)*2^n)
        disp('length of input vector must be like (n+1)*2^n');
        return;
    end

    m2 = m/2;
    if(sum(ismember([1,2,3,4,5,6,7,8],m2))==0)
        disp('Order of spline must be 2, 4, 6, 8, 10, 12, 14 or 16');
        return;
    end
    v=zeros(1,(n+1)*2^n);
    w=v;

    %step 1
    for r=0:n
        for l=0:2^(n-r)-1
            i1=isp(n,r,0,l);
            i2=isp(n,r,2^r-1,l);
            IND=i1:2^(n-r):i2;
            v(IND)=aspline_int(a(IND),m);
        end
    end
    
    %step 2
    u=v;
    for r=0:n-1
        for k=0:2^r-1
            for l=0:2^(n-r-1)-1
                i1=isp(n,r+1,2*k,l);
                for t=0:2*m2
                    i2=isp(n,r,k,mod(t+2*l,2^(n-r)));
                    u(i1)=u(i1)-p(t+1)*v(i2);
                end
            end
        end
    end
    
    %step 3
    for r=0:n
        for k=0:2^r-1
            i1=isp(n,r,k,0);
            w(i1 : i1-1+2^(n-r))=aspline_int(u(i1 : i1-1+2^(n-r)),m);
        end
    end

    if nargin==5
        T=zeros(1,(n+2)*2^(n-1));
        for k=0:n   
            for m=k:-1:0
                % all level indices l
                l(1)=m;
                l(2)=k-m;  
                for ii=1:ceil(2^(l(1)-1))
                    for jj=1:ceil(2^(l(2)-1))
                        % all space indices for every level
                        if(l(1)==0)
                            q(1)=0;
                        else
                            q(1)=2*ii-1;
                        end
                        if(l(2)==0)
                            q(2)=0;
                        else
                            q(2)=2*jj-1;
                        end
                        ind=hcfft_index(l, q, 2, k, in);
                        for kk=l(1):n-l(2)
                            T(ind)=T(ind)+w(isp(n, kk, 2^(kk-l(1))*q(1),2^(n-kk-l(2))*q(2) ));
                        end
                    end
                end
            end
        end
        S=T;
    else
        S=w;
    end
end

function [ T ]=abool_int_3d(v, m, n, p, in )

%    m2=m/2;
    if(sum(ismember([2,4,6,8,10,12,14,16],m))==0)
        disp('Order of spline has to be 2,4,6,8,10,12,14 or 16');
        return;
    end

    % test length of v
    if(length(v)~=(n+1)*(n+2)*2^(n-1))
        disp('length of input-vector has to be (n+1)*(n+2)*2^(n-1)');
        return;
    end
    

    %step 1 = step 3'
    for j=0:n
        for k3=0:2^(n-j)-1
            i1=isp3(n,j,0,0,0,k3);
            i2=isp3(n,j,j,2^j-1,0,k3);
            IND=i1:2^(n-j):i2;
            a(IND)=abool_int_2d(v(IND),m,j,p);
        end
    end
    v=a;
    u=v;
    
    %step 2 = step 2'
    for j=0:n-1
        for j1=0:j
            for k1=0:2^j1-1
                for k2=0:2^(j-j1)-1
                    for k3=0:2^(n-j-1)-1
                        i1=isp3(n,j+1,j1,k1,2*k2,k3);
                        for s=0:m
                            i2=isp3(n,j,j1,k1,k2,mod(s+2*k3, 2^(n-j)));
                            u(i1)=u(i1)-p(s+1)*v(i2);
                        end
                    end
                end
            end
        end
    end
    
    %step 3 = step 1'
    w(1:2^n)=aspline_int(u(1:2^n),m);
    for j=1:n
        i1=j*(j+1)*2^(n-1);
        for jj=0:(j+1)*2^j-1
            w(i1+jj*2^(n-j)+1:i1+(jj+1)*2^(n-j))=aspline_int(u(i1+jj*2^(n-j)+1:i1+(jj+1)*2^(n-j)),m);
        end
    end
    
    %step 4=step 0': change format of sparse grid vector to our needs
    T=zeros(1,2^n*(n+1)+2^(n-3)*n*(n-1));
    for k=0:n  
        for a=0:k
            for b=0:(k-a)
                l(1)=a;
                l(2)=b;
                l(3)=k-a-b;
                for i1=1:ceil(2^(l(1)-1))     %Durchlaufen aller Punkte zum Level l
                    for i2=1:ceil(2^(l(2)-1))
                        for i3=1:ceil(2^(l(3)-1))
                            if(l(1)==0)
                                q(1)=0;
                            else
                                q(1)=2*i1-1;
                            end
                            if(l(2)==0)
                                q(2)=0;
                            else
                                q(2)=2*i2-1;
                            end
                            if(l(3)==0)
                                q(3)=0;
                            else
                                q(3)=2*i3-1;
                            end
            
                            ind=hcfft_index(l, q, 3, k, in);
                            for r3=l(3):n-l(1)-l(2)
                                for r2=l(2):n-l(1)-r3
                                    r1=n-r2-r3;
                                    T(ind)=T(ind)+w(isp3(n, r1+r2, r1, 2^(r1-l(1))*q(1), 2^(r2-l(2))*q(2), 2^(r3-l(3))*q(3)));
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
end


function [ a ] = spline_int( y, m )
% interpolates function values y with spline scaling functions of order m=2r
% 1d function
%
% [a] = spline_int(y,m)
%
% y     = 2^k equispaced function values
% m     = spline order (2,4,6,8,10,12,14 or 16)

    r = m/2;
    if((r~=1) && (r~=2) && (r~=3) && (r~=4) && (r~=5) && (r~=6) && (r~=7) && (r~=8))
        disp('Order of spline must be 2, 4, 6, 8, 10, 12, 14 or 16');
        return;
    end
    dj=length(y);
    n=round(log2(dj));
    if(dj~=2^n)
        disp('length of input vector must be power of 2');
        return;
    end

%initializing some values 
    if(r==1)
        N1=1;
        x=2;
    elseif(r==2)
        N1=1/6;
        x=-4;
    elseif(r==3)
        N1=1/120;
        x=[-13+sqrt(105) -13-sqrt(105)];
    elseif(r==4)
        N1=1/5040;
        x=4*sqrt(301)*cos(1/3*acos(-738/43/sqrt(301)) + [0 2*pi/3 4*pi/3])-40;
        %x=[-109.31435788702851150, -8.2821820468535975470, -2.4034600661178909568];
    elseif(r==5)
        N1=1/362880;
        x=[-471.40962886770841672, -23.179262606297964701, -5.1583672319746813788, -2.2527412940189371967];
    elseif(r==6)
        N1=1/39916800;
        x=[-1958.6436262332338274, -60.006013001985442377, -11.230629237116218333, -3.9462148730617698088, -2.1735166546027420446];
    elseif(r==7)
        N1=1/6227020800;
        x=[-7993.5980068703959182, -148.41803776763827574, -23.183934089907887197, -7.3382674609716999715, -3.3351435177694775500, -2.1266102933167413553];
    elseif(r==8)
        N1=1/1307674368000;
        x=[-32325.133514538724068, -356.99884595613998924, -45.994397117240053369, -13.249820411888182941, -5.5478723417566693683, -2.9790426780923634390, -2.0965069561586741292];
    end;
    lambda=x/2+sqrt((x/2).^2-1);
    
    u=zeros(2*r-1, dj);
%1st step
    lo=mod(r,dj);
    u(1,1:dj-lo)=y((1+lo):dj)/N1;
    u(1,dj-lo+1:dj)=y(1:lo)/N1;
%2nd step
    for nu=1:r-1
    %(a)
        u(2*nu, 1)=1/(1-lambda(nu)^dj)*(u(2*nu-1, 2:dj) * lambda(nu).^(0:dj-2)' + lambda(nu)^(dj-1)*u(2*nu-1, 1));
    %(b)
        u(2*nu, dj)=u(2*nu-1, 1) + lambda(nu)*u(2*nu, 1);
        for k=dj-1:-1:2
            u(2*nu, k) = u(2*nu-1, k+1) + lambda(nu)*u(2*nu, k+1);
        end
    %(c)
        u(2*nu+1, 1)=-1/(1-lambda(nu)^dj)*(u(2*nu, dj:-1:1) * lambda(nu).^(1:dj)');
    %(d)
        for k=0:dj-2
            u(2*nu+1, k+2) = lambda(nu)*(u(2*nu+1, k+1) - u(2*nu, k+1));
        end
    end
%3rd step
    a=zeros(size(y));
    a(1:dj)=u(2*r-1, 1:dj);
end


function [ a ] = aspline_int( y, m )
% adjoint algorithm of spline_int, which
% interpolates function values y with spline scaling functions of order m=2r
%
%
% [a] = aspline_int(y,m)
%
% y     = spline coefficients
% m     = spline order

    r = m/2;
    if((r~=1) && (r~=2) && (r~=3) && (r~=4) && (r~=5) && (r~=6) && (r~=7) && (r~=8))
        disp('Order of spline must be 2, 4, 6, 8, 10, 12, 14 or 16');
        return;
    end
    dj=length(y);
    n=round(log2(dj));
    if(dj~=2^n)
        disp('length of input vector must be power of 2');
        return;
    end

%initializing some values 
    if(r==1)
        N1=1;
        x=2;
    elseif(r==2)
        N1=1/6;
        x=-4;
    elseif(r==3)
        N1=1/120;
        x=[-13+sqrt(105) -13-sqrt(105)];
    elseif(r==4)
        N1=1/5040;
        x=4*sqrt(301)*cos(1/3*acos(-738/43/sqrt(301)) + [0 2*pi/3 4*pi/3])-40;
        %x=[-109.31435788702851150, -8.2821820468535975470, -2.4034600661178909568];
    elseif(r==5)
        N1=1/362880;
        x=[-471.40962886770841672, -23.179262606297964701, -5.1583672319746813788, -2.2527412940189371967];
    elseif(r==6)
        N1=1/39916800;
        x=[-1958.6436262332338274, -60.006013001985442377, -11.230629237116218333, -3.9462148730617698088, -2.1735166546027420446];
    elseif(r==7)
        N1=1/6227020800;
        x=[-7993.5980068703959182, -148.41803776763827574, -23.183934089907887197, -7.3382674609716999715, -3.3351435177694775500, -2.1266102933167413553];
    elseif(r==8)
        N1=1/1307674368000;
        x=[-32325.133514538724068, -356.99884595613998924, -45.994397117240053369, -13.249820411888182941, -5.5478723417566693683, -2.9790426780923634390, -2.0965069561586741292];
    end;
    lambda=x/2+sqrt((x/2).^2-1);
    
    u=zeros(2*r-1, dj);
%1st step
    lo=mod(r,dj);
    u(1,(1+lo):dj)=y(1:dj-lo)/N1; %here only change for adjoint algorithm
    u(1,1:lo)=y(dj-lo+1:dj)/N1;   %and here
%2nd step
    for nu=1:r-1
    %(a)
        u(2*nu, 1)=1/(1-lambda(nu)^dj)*(u(2*nu-1, 2:dj) * lambda(nu).^(0:dj-2)' + lambda(nu)^(dj-1)*u(2*nu-1, 1));
    %(b)
        u(2*nu, dj)=u(2*nu-1, 1) + lambda(nu)*u(2*nu, 1);
        for k=dj-1:-1:2
            u(2*nu, k) = u(2*nu-1, k+1) + lambda(nu)*u(2*nu, k+1);
        end
    %(c)
        u(2*nu+1, 1)=-1/(1-lambda(nu)^dj)*(u(2*nu, dj:-1:1) * lambda(nu).^(1:dj)');
    %(d)
        for k=0:dj-2
            u(2*nu+1, k+2) = lambda(nu)*(u(2*nu+1, k+1) - u(2*nu, k+1));
        end
    end
%3rd step
    a=zeros(size(y));
    a(1:dj)=u(2*r-1, 1:dj);
end




function ind=isp(n,r,mu,l)
    ind=r*2^n+mu*2^(n-r)+l+1;
end

function ind=isp3(n, j, j1, k, l, m)
    ind=j*(j+1)*2^(n-1) + (j1*2^j + k*2^(j-j1) + l)*2^(n-j) + m+1;
end

